@@ -3,6 +3,7 @@
 #include "ugg_bsm.h"
 #include "ung_bsm.h"
 #include "ung_svm.h"
+#include "ung_ar1.h"
 #include "mgg_ssm.h"
 #include "nlg_ssm.h"
 
@@ -30,6 +31,12 @@ Rcpp::List gaussian_approx_model(const Rcpp::List& model_,
     return Rcpp::List::create(Rcpp::Named("y") = approx_model.y,
       Rcpp::Named("H") = approx_model.H);
   } break;
+  case 4: {
+    ung_ar1 model(clone(model_), 1);
+    ugg_ssm approx_model = model.approximate(mode_estimate, max_iter, conv_tol);
+    return Rcpp::List::create(Rcpp::Named("y") = approx_model.y,
+      Rcpp::Named("H") = approx_model.H);
+  } break;
   default: 
     return Rcpp::List::create(Rcpp::Named("y") = 0, Rcpp::Named("H") = 0);
   }
